@@ -4,9 +4,9 @@ import re
 import logging
 from collections import OrderedDict
 
-import six
 from ruamel.yaml import YAML
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
+from operator_manifest.helpers import validate_with_schema
 
 
 yaml = YAML()
@@ -42,10 +42,9 @@ def is_list(obj):
 
 def is_str(obj):
     """
-    Check if object is a string or bytes. On python 3, checking for string
-    would be sufficient, but on python 2, it may not be.
+    Check if object is a string or bytes.
     """
-    return isinstance(obj, (bytes, six.text_type))
+    return isinstance(obj, (bytes, str))
 
 
 class PullspecRegex(object):
@@ -306,11 +305,7 @@ class OperatorCSV(object):
             implementation will be used. For more information, see the
             default_pullspec_heuristic() function in this module.
         """
-        try:
-            if data.get("kind") != OPERATOR_CSV_KIND:
-                raise NotOperatorCSV("Not a ClusterServiceVersion")
-        except AttributeError:
-            raise NotOperatorCSV("File does not contain a YAML object")
+        check_csv(data, 'schemas/csv_schema.json')
         self.path = path
         self.data = data
         self._pullspec_heuristic = pullspec_heuristic
@@ -386,6 +381,10 @@ class OperatorCSV(object):
         """
         named_pullspecs = self._named_pullspecs()
 
+        if not named_pullspecs:
+            log.info("No pullspecs, skipping updates of relatedImages section")
+            return
+
         by_name = OrderedDict()
         conflicts = []
 
@@ -425,12 +424,27 @@ class OperatorCSV(object):
         return pullspecs
 
     def _related_image_pullspecs(self):
+        """Get the pullspecs from spec.relatedImages section
+
+        :return: a list of pullspecs. It could be an empty if no spec.RelatedImage section.
+        :rtype: list[RelatedImage]
+        """
         related_images_path = ("spec", "relatedImages")
         return [
             RelatedImage(r)
             for r in chain_get(self.data, related_images_path, default=[])
         ]
 
+    def get_related_image_pullspecs(self):
+        """Get the related image pullspecs
+
+        :return: a list of related images pullspecs. It could be an empty list
+            if this CSV file does not have spec.relatedImages section.
+        :rtype: list[ImageName]
+        """
+        return [ImageName.parse(related_image.image)
+                for related_image in self._related_image_pullspecs()]
+
     def _deployments(self):
         deployments_path = ("spec", "install", "spec", "deployments")
         return chain_get(self.data, deployments_path, default=[])
@@ -542,13 +556,23 @@ class OperatorManifest(object):
     Currently, only ClusterServiceVersion files are considered relevant.
     """
 
-    def __init__(self, files):
+    def __init__(self, csv_file):
         """
         Initialize an OperatorManifest
 
-        :param files: list of OperatorCSVs
+        :param csv_file: OperatorCSV
         """
-        self.files = files
+        assert isinstance(csv_file, OperatorCSV)
+        self._csv_file = csv_file
+        self.files = [self._csv_file]  # BW compat
+
+    @property
+    def csv(self):
+        """
+        :return: ClusteredServiceVersion object of the operator manifests
+        :rtype: OperatorCSV
+        """
+        return self._csv_file
 
     @classmethod
     def from_directory(cls, path, **kwargs):
@@ -563,7 +587,16 @@ class OperatorManifest(object):
             raise RuntimeError("Path does not exist or is not a directory: {}".format(path))
         yaml_files = cls._get_yaml_files(path)
         operator_csvs = list(cls._get_csvs(yaml_files, **kwargs))
-        return cls(operator_csvs)
+        if not operator_csvs:
+            raise ValueError("Missing ClusterServiceVersion in operator manifests")
+        if len(operator_csvs) > 1:
+            raise ValueError(
+                "Operator bundle may contain only 1 CSV file, but contains more: {}".format(
+                    ", ".join(csv.path for csv in operator_csvs)
+                )
+            )
+
+        return cls(operator_csvs[0])
 
     @classmethod
     def _get_yaml_files(cls, dirpath):
@@ -698,3 +731,12 @@ def chain_get(d, path, default=None):
         except (IndexError, KeyError):
             return default
     return obj
+
+
+def check_csv(data, schema):
+    try:
+        if data.get("kind") != OPERATOR_CSV_KIND:
+            raise NotOperatorCSV("Not a ClusterServiceVersion")
+    except AttributeError as exc:
+        raise NotOperatorCSV("File does not contain a YAML object") from exc
+    validate_with_schema(data, schema)
